from typing import Union

from xybase.reflection import get_cls_annotations, get_cls_name


class Converter:
    custom_cls_prefix: list[str]

    def __init__(self, custom_cls_prefix: Union[str, list[str]] = None):
        if custom_cls_prefix:
            self.custom_cls_prefix = custom_cls_prefix if isinstance(custom_cls_prefix, list) else [custom_cls_prefix]

    def convert(self, obj, cls):
        """ 将对象转换为其他类型 """
        if obj is None:
            return obj

        if isinstance(obj, list):
            if not obj:
                return obj

            new_items = []
            for item in obj:
                new_items.append(self.convert(item, cls))
            return new_items

        old_obj = obj if isinstance(obj, dict) else obj.__dict__
        new_obj = cls()
        annotations = get_cls_annotations(cls)

        for k in old_obj:
            v = old_obj[k]
            if v is not None and k in annotations:
                attr_cls_name = get_cls_name(annotations[k])
                if self.custom_cls_prefix and list(
                        filter(lambda b: attr_cls_name.startswith(b), self.custom_cls_prefix)):
                    setattr(new_obj, k, self.convert(v, annotations[k]))
                    continue
                setattr(new_obj, k, old_obj[k])
        return new_obj



from xybase.converter import Converter


class B:
    age: int


class A:
    name: str
    b: B


if __name__ == '__main__':
    c = Converter('__main__')
    a: A = c.convert({'name': 'x', 'b': {'age': 1}}, A)
    print(a.b.age)



def get_cls_name(cls: type):
    """ 获取类全名(包含包名) """
    return str(cls).replace('<class \'', '').replace('\'>', '')


def get_cls_annotations(cls: type):
    """ 获取类的所有字段(包括继承父类的) """
    annotations = {}
    __get_cls_annotations__(cls, annotations)
    return annotations


def __get_cls_annotations__(cls: type, annotations: dict):
    if cls.__base__ == object:
        annotations.update(cls.__annotations__)
        return

    for base in cls.__bases__:
        __get_cls_annotations__(base, annotations)



from typing import Union


def write(f, content: Union[bytes, str]):
    mode = 'w' if isinstance(content, str) else 'wb'
    with open(f, mode=mode) as w:
        w.write(content)


def read_bytes(f) -> bytes:
    with open(f, mode='rb') as r:
        return r.read()


def read_lines(f) -> list[str]:
    with open(f, mode='r') as r:
        return r.read().splitlines()



def singleton(cls):
    """ 单例模式 """
    instances = {}

    def get_instance(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return get_instance


def group(items: list, by: str):
    """ 简单分组函数 """
    result: dict[str, list] = {}
    for item in items:
        d = item if isinstance(item, dict) else item.__dict__
        if d[by] not in result:
            result[d[by]] = []
        result[d[by]].append(item)
    return result



from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from smtplib import SMTP_SSL
from typing import Union


class EmailServer:
    smtp: SMTP_SSL

    def __init__(self, host: str, port: int, user: str, password: str):
        self.smtp = SMTP_SSL(host, port)
        self.smtp.login(user, password)

    def quit(self):
        self.smtp.quit()


class Email:
    msg: MIMEMultipart
    smtp: SMTP_SSL

    def __init__(self, server: Union[EmailServer, SMTP_SSL]):
        self.msg = MIMEMultipart()
        self.smtp = server if isinstance(server, SMTP_SSL) else server.smtp

    def from_(self, from_: str):
        """ 标题 """
        self.msg['from'] = from_
        return self

    def subject(self, subject: str):
        """ 副标题 """
        self.msg['subject'] = subject
        return self

    def to(self, to: Union[str, list[str]]):
        """ 收件人列表 """
        self.msg['to'] = to if isinstance(to, str) else ','.join(to)
        return self

    def text(self, text: str, charset='utf-8'):
        """ 文本 """
        self.msg.attach(MIMEText(text, _charset=charset))
        return self

    def html(self, html: str, charset='utf-8'):
        """ HTML """
        self.msg.attach(MIMEText(html, 'html', charset))
        return self

    def attach(self, source: Union[str, bytes], filename: str = None):
        """ 附件 """
        att = MIMEApplication(open(source, 'rb').read() if isinstance(source, str) else source)
        if not filename and isinstance(source, str):
            paths = source.split('/')
            filename = paths[len(paths) - 1]
        att.add_header('Content-Disposition', 'attachment', filename=filename)
        self.msg.attach(att)
        return self

    def send(self, to_addrs: Union[str, list[str]]):
        """ 发送邮件，to_addrs:收件人地址列表, from_addr:与服务器登录用户邮箱地址一致 """
        self.smtp.sendmail(from_addr=self.smtp.user, to_addrs=to_addrs, msg=self.msg.as_string())
